@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2004  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -351,13 +351,10 @@ xfade_load_config()
   }
   else
     DEBUG(("[crossfade] load_config: error loading config, using defaults\n"));
- 
-  if((filename = g_strconcat(g_get_home_dir(), "/.xmms/xmms-crossfade-presets", NULL))) {
-    scan_presets(filename);
-    g_free(filename);
-  }
-  else
-    DEBUG(("[crossfade] load_config: g_strconcat() failed!\n"));
+
+  filename = g_strconcat(g_get_home_dir(), "/.xmms/xmms-crossfade-presets", NULL);
+  // scan_presets(filename);
+  g_free(filename);
 }
 
 void
@@ -945,7 +942,7 @@ scan_plugins(GtkWidget *option_menu, gchar *selected)
     OutputPlugin *op = (OutputPlugin *)list->data;
     GtkWidget  *item = gtk_menu_item_new_with_label(op->description);
     
-    if(op == get_oplugin_info())  /* disable selecting ourselves */
+    if(op == get_crossfade_oplugin_info())  /* disable selecting ourselves */
       gtk_widget_set_sensitive(item, FALSE);
     else {
       if(def_index == -1) def_index = index;
@@ -1109,6 +1106,11 @@ config_effect_plugin_cb(GtkWidget *widget, gint index)
   /* update gui */
   SET_SENSITIVE("ep_configure_button", ep && (ep->configure != NULL)); 
   SET_SENSITIVE("ep_about_button",     ep && (ep->about != NULL));
+
+  /* 0.3.5: apply effect config immediatelly */
+  if(config->ep_name) g_free(config->ep_name);
+  config->ep_name = g_strdup(cfg->ep_name);
+  xfade_realize_ep_config();
 }
 
 void on_ep_configure_button_clicked(GtkButton *button, gpointer user_data)
@@ -1125,6 +1127,13 @@ void on_ep_about_button_clicked(GtkButton *button, gpointer user_data)
   ep->about();
 }
 
+void on_ep_enable_check_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+  /* 0.3.5: apply effect config immediatelly */
+  config->ep_enable = cfg->ep_enable = GET_TOGGLE("ep_enable_check");
+  xfade_realize_ep_config();
+}
+
 /*-- volume normalizer ------------------------------------------------------*/
 
 void check_effects_dependencies()
@@ -1775,7 +1784,6 @@ void on_config_apply_clicked(GtkButton *button, gpointer user_data)
   /* output method: none: */
 
   /* effects: pre-mixing effect plugin */
-  cfg->ep_enable = GET_TOGGLE("ep_enable_check");
 
   /* effects: volume normalizer */
   cfg->volnorm_target = GET_SPIN  ("volnorm_target_spin");
@@ -1816,6 +1824,7 @@ void on_config_apply_clicked(GtkButton *button, gpointer user_data)
   if(config->oss_alt_mixer_device) g_free(config->oss_alt_mixer_device);
   if(config->op_config_string)     g_free(config->op_config_string);
   if(config->op_name)              g_free(config->op_name);
+  if(config->ep_name)              g_free(config->ep_name);
 
   /* copy current settings (dupping the strings) */
   *config = *cfg;
@@ -2067,7 +2076,7 @@ void xfade_about()
   if(!about_win) {
     gchar *about_text = 
       "XMMS Crossfade Plugin "VERSION"\n"
-      "Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>\n"
+      "Copyright (C) 2000-2004  Peter Eisenlohr <peter@eisenlohr.org>\n"
       "\n"
       "based on the original OSS Output Plugin  Copyright (C) 1998-2000\n"
       "Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies\n"
