@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2004  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -97,14 +97,16 @@ static OutputPlugin xfade_op =
 };
 
 /* internal prototypes */
-static void  load_symbols   ();
-static gint  open_output    ();
-static void  buffer_reset   (buffer_t *buf, config_t *cfg);
-static void *buffer_thread_f(void *arg);
-static void  sync_output    ();
+static void  load_symbols    ();
+static void  output_list_hack();
+static gint  open_output     ();
+static void  buffer_reset    (buffer_t *buf, config_t *cfg);
+static void *buffer_thread_f (void *arg);
+static void  sync_output     ();
 
 /* special XMMS symbols (dynamically looked up, see xfade_init) */
-static gboolean  *playlist_get_info_going      = NULL;  /* XMMS */
+static gboolean  *xmms_playlist_get_info_going = NULL;  /* XMMS */
+static gboolean  *xmms_is_quitting             = NULL;  /* XMMS */
 static gboolean (*input_stopped_for_restart)() = NULL;  /* XMMS */
 
 #ifdef HAVE_CTOR
@@ -140,6 +142,8 @@ static gint     output_flush_time = 0;
 static gint64   output_written    = 0;
        gint64   output_streampos  = 0;
 
+static gboolean finishing = FALSE;  /* TRUE after fini() has been called */
+
 static gchar zero_4k[4096];
 
 #ifdef VOLUME_NORMALIZER
@@ -219,6 +223,11 @@ OutputPlugin *get_oplugin_info()
   return &xfade_op;
 }
 
+OutputPlugin *get_crossfade_oplugin_info()
+{
+  return &xfade_op;
+}
+
 static gint
 effect_list_f(gconstpointer a, gconstpointer b)
 {
@@ -233,11 +242,11 @@ open_output_f(gpointer data)
 {
   DEBUG(("[crossfade] open_output_f: pid=%d\n", getpid()));
   open_output();
-  return FALSE;  // FALSE = 'do not call me again'
+  return FALSE;  /* FALSE = 'do not call me again' */
 }
 
 void
-xfade_realize_config()  /* also called by xfade_init() */
+xfade_realize_ep_config()
 {
   GList        *list, *element;
   EffectPlugin *ep = NULL;
@@ -248,6 +257,13 @@ xfade_realize_config()  /* also called by xfade_init() */
       ep = element->data;
 
   effect_set_plugin(&effect_context, ep);
+}
+
+void
+xfade_realize_config()  /* also called by xfade_init() */
+{
+  /* realize effect plugin config */
+  xfade_realize_ep_config();
 
 #ifdef VOLUME_NORMALIZER
   /* update target volume normalizer target rms */
@@ -488,6 +504,9 @@ xfade_init()
 
   /* load any dynamic linked symbols */
   load_symbols();
+  
+  /* HACK: make sure we are at the beginning of XMMS' output plugin list */
+  output_list_hack();
 
   /* realize config -- will also setup the pre-mixing effect plugin */
   xfade_realize_config();
@@ -512,36 +531,72 @@ static void load_symbols()
   DEBUG((!(error = dlerror()) ? " found\n" : " not found\n"));
 
   DEBUG(("[crossfade] load_symbols: playlist_get_info_going:"));
-  playlist_get_info_going = dlsym(handle, "playlist_get_info_going");
+  xmms_playlist_get_info_going = dlsym(handle, "playlist_get_info_going");
   DEBUG((!(error = dlerror()) ? " found\n" : " not found\n"));
   
+  DEBUG(("[crossfade] load_symbols: is_quitting:"));
+  xmms_is_quitting = dlsym(handle, "is_quitting");
+  DEBUG((!(error = dlerror()) ? " found\n" : " not found\n"));
+
   dlclose(handle);
 #endif
 }
 
+/*
+  HACK: Try to move ourselves to the beginning of XMMS output plugin list,
+        so that we will be freed first when XMMS is quitting. This way, we
+        avoid the segfault when using ALSA as the output plugin.     
+*/
+static void output_list_hack()
+{
+  GList *output_list = get_output_list();
+
+  int i0 = g_list_index(output_list, &xfade_op), i1;
+
+  GList *first = g_list_first(output_list);
+  GList *xfade = g_list_find (output_list, &xfade_op);
+  xfade->data  = first->data;
+  first->data  = &xfade_op;
+
+  i1 = g_list_index(output_list, &xfade_op);
+  if (i0 != i1)
+    DEBUG(("[crossfade] output_list_hack: crossfade moved from index %d to %d\n", i0, i1));
+}
+
 void fini()
 {
   DEBUG(("[crossfade]\n"));
   DEBUG(("[crossfade] fini: cleanup:\n"));
 
-  /* signal buffer_thread to stop */
+  /* wait for buffer_thread to stop */
   pthread_mutex_lock(&buffer_mutex);
-#if 0
-  stopped = TRUE;
-#else
+
   /* reset last_close so that the buffer thread will
-   * not unecessarily wait for a timeout. */
+   * not unnecessarily wait for a timeout. */
   memset(&last_close, 0, sizeof(last_close));
-#endif
+
+  /* HACK: tell buffer thread not to close output by itself */
+  finishing = TRUE;
   paused = FALSE;
 
   /* wait for buffer thread to clean up and terminate */
-  while(output_opened) {
+  DEBUG(("[crossfade] fini: cleanup: waiting for buffer thread...\n"));
+  while(output_opened && finishing) {
     pthread_mutex_unlock(&buffer_mutex);
     xmms_usleep(10000);
     pthread_mutex_lock(&buffer_mutex);
   }
-  /* xmms_usleep(10000); */
+  DEBUG(("[crossfade] fini: cleanup: waiting for buffer thread... done\n"));
+
+  /* HACK: 0.3.5: close output plugin from THIS thread */
+  if(output_opened) {
+    DEBUG(("[crossfade] fini: cleanup: closing audio...\n"));
+    if(the_op->close_audio) the_op->close_audio();
+    DEBUG(("[crossfade] fini: cleanup: closing audio... done\n"));
+    g_free(buffer->data);
+    output_opened = FALSE;
+  }
+  DEBUG(("[crossfade] fini: cleanup: done\n"));
 
   /* unlock and free buffer mutex */
   pthread_mutex_unlock (&buffer_mutex);
@@ -926,6 +981,18 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
   }
   last_filename = g_strdup(file); 
 
+#if 0
+  /* FIXME: finish this */
+  /* Check if this is a short song. */
+  if(fade_config == &config->fc[FADE_CONFIG_XFADE]) {
+    DEBUG(("*** XFADE:\n"));
+    int current_length = playlist_get_current_length();
+    DEBUG(("*** length=%d\n", current_length));
+    if (current_length < 30*1000)
+      fade_config = &config->fc[FADE_CONFIG_ALBUM];
+  }
+#endif
+
 #ifdef VOLUME_NORMALIZER
   /* turn off volume normalizing per default, until we get a target RMS for this song */
   volume_set_active(&volume_context, FALSE);
@@ -1059,6 +1126,7 @@ xfade_open_audio(AFormat fmt, int rate, int nch)
 
     /* flush output plugin */
     the_op->flush(0);
+    output_streampos = 0;
 
     /* flush buffer */
     buffer_reset(buffer, config);
@@ -1398,7 +1466,7 @@ static void sync_output()
   else
     DEBUG(("[crossfade] sync_output: ... done (%ld ms)\n", total));
 }
-	  
+
 void *
 buffer_thread_f(void *arg)
 {
@@ -1811,14 +1879,19 @@ buffer_thread_f(void *arg)
   /* cleanup: close output */
   if(output_opened) {
     xfade_stop_monitor();
-    if(the_op->close_audio) the_op->close_audio();
-    g_free(buffer->data);
-    output_opened = FALSE;
-    DEBUG(("[crossfade] buffer_thread_f: output closed\n"));
+
+    if(!finishing) {
+      if(the_op->close_audio) the_op->close_audio();
+      g_free(buffer->data);
+      output_opened = FALSE;
+    }
   }
   else
     DEBUG(("[crossfade] buffer_thread_f: output already closed!\n"));
 
+  /* acknowledge 'finishing' (see fini()) */
+  finishing = FALSE;
+
   /* ----------------------------------------------------------------------- */
 
   /* unlock buffer */
@@ -1828,7 +1901,7 @@ buffer_thread_f(void *arg)
   DEBUG(("[crossfade] buffer_thread_f: thread finished\n"));
   pthread_exit(0);
 
-  return NULL;  // avoid compiler warning
+  return NULL;  /* avoid compiler warning */
 }
 
 void
@@ -1864,16 +1937,22 @@ xfade_close_audio()
 	stopped = TRUE;
     }
 
-    /* HACK: if playlist_get_info_going is not true here,
+    /* HACK: If playlist_get_info_going is not true here,
      *       XMMS is about to exit. In this case, we stop
      *       the buffer thread before returning from this
      *       function. Otherwise, SEGFAULT may occur when
      *       XMMS tries to cleanup an output plugin which
      *       we are still using.
+     *
+     * NOTE: This hack has become obsolete as of 0.3.5.
+     *       See output_list_hack().
      */
-    if(playlist_get_info_going && !*playlist_get_info_going) {
+#if 0
+    if((xmms_playlist_get_info_going && !*xmms_playlist_get_info_going) ||
+       (xmms_is_quitting             &&  *xmms_is_quitting)) {
       DEBUG(("[crossfade] close: stop (about to quit)\n"))
   
+#if 1
       /* wait for buffer thread to clean up and terminate */
       stopped = TRUE;
       while(output_opened) {
@@ -1881,8 +1960,10 @@ xfade_close_audio()
 	xmms_usleep(10000);
 	pthread_mutex_lock(&buffer_mutex);
       }
+#endif
     }
     else
+#endif
       DEBUG(("[crossfade] close: stop\n"));
 
     fade_config = &config->fc[FADE_CONFIG_MANUAL];
@@ -1983,6 +2064,7 @@ xfade_flush(gint time)
     /* flush output plugin */
     the_op->flush(time);
     output_flush_time = time;
+    output_streampos  = MS2B(time);
 
     /* flush buffer, disable leading gap killing */
     buffer_reset(buffer, config);
@@ -2086,7 +2168,7 @@ xfade_pause(short p)
       buffer->silence     = out_len;
       buffer->silence_len = silence_len;
       buffer->pause       = out_len + silence_len;
-      paused = FALSE;  // (!) will be set to TRUE in buffer_thread_f
+      paused = FALSE;  /* (!) will be set to TRUE in buffer_thread_f */
     }
     else {
       the_op->pause(1);
