@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2004  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -331,6 +331,7 @@
 
 /* get plugin info (imported by XMMS) */
 OutputPlugin *get_oplugin_info();
+OutputPlugin *get_crossfade_oplugin_info();
 
 /* xmms internal prototypes */ /* XMMS */
 gint     ctrlsocket_get_session_id();
@@ -343,8 +344,9 @@ gint     playlist_get_current_length();
 gchar *playlist_get_filename(gint pos);
 #endif
 
-/* config change callback */
+/* config change callbacks */
 void xfade_realize_config();
+void xfade_realize_ep_config();
 
 
 typedef struct
