@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2004  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -36,12 +36,42 @@ effect_init(effect_context_t *ec,
   effect_set_plugin(ec, ep);
 }
 
+static char *
+plugin_name(EffectPlugin *ep)
+{
+  return ep->description ? ep->description : "<unnamed>";
+}
+
+
 void
 effect_set_plugin(effect_context_t *ec,
 		  EffectPlugin     *ep)
 {
+  if(ec->use_xmms_plugin && (ep == EFFECT_USE_XMMS_PLUGIN)) return;
+  if(ec->ep == ep) return;
+
+  if(ec->last_ep) {
+    if(ec->last_ep->cleanup) {
+      DEBUG(("[crossfade] effect: \"%s\" deselected, cleanup\n", plugin_name(ec->last_ep)));
+      ec->last_ep->cleanup();
+    }
+    else
+      DEBUG(("[crossfade] effect: \"%s\" deselected\n", plugin_name(ec->last_ep)));
+  }
+
   ec->use_xmms_plugin = (ep == EFFECT_USE_XMMS_PLUGIN);
-  ec->ep = ec->use_xmms_plugin ? NULL : ep;
+  ec->ep        = ec->use_xmms_plugin ? NULL : ep;
+  ec->last_ep   = ec->ep;
+  ec->is_active = FALSE;
+
+  if(ec->ep) {
+    if(ec->ep->init) {
+      DEBUG(("[crossfade] effect: \"%s\" selected, init\n", plugin_name(ec->ep)));
+      ec->ep->init();
+    }
+    else
+      DEBUG(("[crossfade] effect: \"%s\" selected\n", plugin_name(ec->ep)));
+  }
 }
 
 gint
@@ -55,11 +85,12 @@ effect_flow(effect_context_t *ec,
 
   if(ec->use_xmms_plugin ? effects_enabled() : (ec->ep != NULL)) {
     ep = ec->use_xmms_plugin ? get_current_effect_plugin() : ec->ep;
+
     if(ep != ec->last_ep) {
       DEBUG(("[crossfade] effect: plugin: \"%s\"%s\n",
-	     ep ? (ep->description ? ep->description : "<unnamed>") : "<none>",
+	     ep ? plugin_name(ep) : "<none>",
 	     ec->use_xmms_plugin ? " (XMMS)" : ""));
-      ec->last_ep = ep;
+      ec->last_ep   = ep;
       ec->is_active = FALSE;
     }
     if(ep == NULL) return length;
@@ -91,10 +122,9 @@ effect_flow(effect_context_t *ec,
 	    ec->is_valid = FALSE;
 	  }
 	  else {
-	    DEBUG(("[crossfade] effect: plugin enabled "
-		   "(fmt=%s rate=%d nch=%d)\n",
-		   format_name(fmt), rate, nch));
 	    ec->is_valid = TRUE;
+	    DEBUG(("[crossfade] effect: plugin enabled "
+		   "(fmt=%s rate=%d nch=%d)\n", format_name(fmt), rate, nch));
 	  }
 	}
 
@@ -109,17 +139,14 @@ effect_flow(effect_context_t *ec,
       }
     }
     else {
-      if(!ec->is_active) {
-	DEBUG(("[crossfade] effect: plugin enabled (default format)\n"));
-	ec->is_active = TRUE;
-      }
+      ec->is_active = TRUE;
       if(ep->mod_samples)
 	length = ep->mod_samples(buffer, length, format->fmt, format->rate, format->nch);
     }
   }
   else if(ec->is_active) {
-    DEBUG(("[crossfade] effect: plugin disabled\n"));
     ec->is_active = FALSE;
+    DEBUG(("[crossfade] effect: plugin disabled\n"));
   }
 
   return length;
