@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2004  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -42,6 +42,7 @@ static GtkLabel    *monitor_position_label;
 static GtkLabel    *monitor_total_label;
 static GtkLabel    *monitor_left_label;
 static GtkLabel    *monitor_output_time_label;
+static GtkLabel    *monitor_output_time_sep;
 static GtkLabel    *monitor_written_time_label;
 
 static gchar       *default_position_str = NULL;
@@ -143,8 +144,9 @@ void xfade_check_monitor_win()
     monitor_position_label      = GTK_LABEL   (lookup_widget(monitor_win, "monpos_position_label"));
     monitor_total_label         = GTK_LABEL   (lookup_widget(monitor_win, "monpos_total_label"));
     monitor_left_label          = GTK_LABEL   (lookup_widget(monitor_win, "monpos_left_label"));
-    monitor_written_time_label  = GTK_LABEL   (lookup_widget(monitor_win, "monpos_written_time_label"));
     monitor_output_time_label   = GTK_LABEL   (lookup_widget(monitor_win, "monpos_output_time_label"));
+    monitor_output_time_sep     = GTK_LABEL   (lookup_widget(monitor_win, "monpos_output_time_separator_label"));
+    monitor_written_time_label  = GTK_LABEL   (lookup_widget(monitor_win, "monpos_written_time_label"));
 
     /* get default strings (displayed when monitor is stopped) */
     if(!default_position_str)     { gtk_label_get(monitor_position_label,     &str); default_position_str     = g_strdup(str); }
@@ -273,7 +275,8 @@ gint xfade_update_monitor(gpointer userdata)
     /*** Output Plugin position */
     if(1 || (monitor_written_time_label && monitor_output_time_label)) {
       if(monitor_closing == CLOSED) {
-	gtk_label_set_text(monitor_output_time_label,  default_output_time_str);
+	gtk_widget_hide(GTK_WIDGET(monitor_output_time_label));
+	gtk_widget_hide(GTK_WIDGET(monitor_output_time_sep));
 	gtk_label_set_text(monitor_written_time_label, default_written_time_str);
       }
       else {
@@ -281,19 +284,29 @@ gint xfade_update_monitor(gpointer userdata)
 
 	/* check for output plugin bug -- diff should always be 0 */
 	gint diff = written_time - (gint)(output_streampos * 1000 / OUTPUT_BPS);
-	g_snprintf(buffer, sizeof(buffer),
-		   output_time < 0 ? "-%d:%02d.%03d" : "%d:%02d.%03d",
-		   ABS((diff/60000)),
-		   ABS((diff/1000) % 60),
-		   ABS((diff)    % 1000));
-	gtk_label_set_text(monitor_output_time_label, buffer);
+	if(diff) {
+	  gtk_widget_show(GTK_WIDGET(monitor_output_time_label));
+	  gtk_widget_show(GTK_WIDGET(monitor_output_time_sep));
+
+	  g_snprintf(buffer, sizeof(buffer),
+		     output_time < 0 ? "-%d:%02d.%03d" : "%d:%02d.%03d",
+		     ABS((diff/60000)),
+		     ABS((diff/1000) % 60),
+		     ABS((diff)    % 1000));
+	  gtk_label_set_text(monitor_output_time_label, buffer);
+	}
+	else {
+	  gtk_widget_hide(GTK_WIDGET(monitor_output_time_label));
+	  gtk_widget_hide(GTK_WIDGET(monitor_output_time_sep));
+	}
 
 	/* written_time */
 	g_snprintf(buffer, sizeof(buffer),
-		   written_time < 0 ? "-%d:%02d.%01d" : "%d:%02d.%01d",
-		   ABS((written_time/60000)),
-		   ABS((written_time/1000) % 60),
-		   ABS((written_time/100)  % 10));
+		   written_time < 0 ? "-%d:%02d:%02d.%01d" : "%d:%02d:%02d.%01d",
+		   ABS((written_time/3600000)),
+		   ABS((written_time/60000) % 60),
+		   ABS((written_time/1000)  % 60),
+		   ABS((written_time/100)   % 10));
 	gtk_label_set_text(monitor_written_time_label, buffer);
       }
     }
