@@ -1,6 +1,6 @@
 /* 
  *  XMMS Crossfade Plugin
- *  Copyright (C) 2000-2003  Peter Eisenlohr <p.eisenlohr@gmx.net>
+ *  Copyright (C) 2000-2004  Peter Eisenlohr <peter@eisenlohr.org>
  *
  *  based on the original OSS Output Plugin
  *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
@@ -321,6 +321,8 @@ oss_open_audio(AFormat fmt, int rate, int nch)
    *       just a crude workaround to avoid complete lockup of XMMS. It is
    *       not perfect since it is theoretically possible that another program
    *       locks the device between the close() and open() calls below. */
+  /*
+  DEBUG(("[xfade-oss] open_audio: WRITE TEST...\n"));
   dsp_fd = open(device_name, O_WRONLY|O_NONBLOCK);
   if(dsp_fd == -1) {
     PERROR("[xfade-oss] open_audio");
@@ -328,6 +330,8 @@ oss_open_audio(AFormat fmt, int rate, int nch)
     return 0;
   }
   close(dsp_fd);
+  DEBUG(("[xfade-oss] open_audio: WRITE TEST... done\n"));
+  */
 
   /* open device (we do need blocking io here!)  */
   dsp_fd = open(device_name, O_WRONLY);
